#!/usr/bin/env python
# BV4614

from sv3common import sv3, util_error, util_information
from time import sleep
import wx

# Global
# DCMotor bit 8 is A run flag and bit 7 is B run flag
EEsettings = [] # saves changes for writing to ee[rpm
i2cadr = None
bus = None

ACK = '\006'
NACK = '\025'

# ------------------------------------------------------------------------------
# start up and close down
# Load frame with default values, from EEPROM
# ------------------------------------------------------------------------------
def onActivate(wnd):
    global EEsettings
    global i2cadr
    global bus
    i2cadr = int(wnd.adr)
    bus = wnd.sp
    # update status bar
    wnd.lblDevice.SetLabel(wnd.barStatus.GetStatusText(0))
    # remove com and baud
    wnd.barStatus.SetStatusText("",1)
    wnd.barStatus.SetStatusText("",2)
    # device ID
    dev = bus.i2c(i2cadr,[0xa1],2)
    v = dev[0]*256
    v += dev[1]
    wnd.barStatus.SetStatusText(str(v),3)
    wnd.barStatus.SetStatusText(str(i2cadr),3) # address
    
    # load eeprom values to 16
    EEsettings = []
    for j in range(16,32):
        v = bus.i2c(i2cadr,[0x90,j],1)
        EEsettings.append(str(v[0]))

    # write all settings to text boxes
    update_EEvalues(wnd)
    # set global power slider to default
    wnd.sldGlobal.SetValue(int(EEsettings[6]))

# Close is important as this is a modal frame
def onClose(wnd,event):
    wnd.MakeModal(False)
    event.Skip()
    
# ------------------------------------------------------------------------------
# this will update the EEPROM information
# ------------------------------------------------------------------------------
def update_EEvalues(wnd):
    wnd.txt0.SetValue(EEsettings[0])
    wnd.txt1.SetValue(EEsettings[1])
    wnd.txt2.SetValue(EEsettings[2])
    wnd.txt3.SetValue(EEsettings[3])
    wnd.txt4.SetValue(EEsettings[4])
    wnd.txt5.SetValue(EEsettings[5])
    wnd.txt6.SetValue(EEsettings[6])
    wnd.txt7.SetValue(EEsettings[7])
    wnd.txt8.SetValue(EEsettings[8])
    if EEsettings[4] == '1':
        util_information("Warning mode is set to 1, set it to \n0 for these commands to work")
        wnd.txt4.SetBackgroundColour(wx.Color(255,0,0))
    else:
        wnd.txt4.SetBackgroundColour(wx.Color(255,255,255))


# ******************************************************************************
# ******************************************************************************
# EVENTS

# ------------------------------------------------------------------------------
def onSoftreset(wnd):
    bus.i2c(i2cadr,[0x95],0)
    wnd.barStatus.SetStatusText("Reset complete")

# ------------------------------------------------------------------------------
def onHardreset(wnd):
    wnd.sp.SetDTR(True)
    sleep(0.1)
    wnd.sp.SetDTR(False)
    wnd.barStatus.SetStatusText("Reset complete")

# ******************************************************************************
# Settings
# ******************************************************************************
def onEEupdate(wnd):
    # need to update all values
    # first update EEValues form text boxes
    EEsettings[0] = wnd.txt0.GetLineText(0)
    EEsettings[1] = wnd.txt1.GetLineText(0)
    EEsettings[2] = wnd.txt2.GetLineText(0)
    EEsettings[3] = wnd.txt3.GetLineText(0)
    EEsettings[4] = wnd.txt4.GetLineText(0)
    EEsettings[5] = wnd.txt5.GetLineText(0)
    EEsettings[6] = wnd.txt6.GetLineText(0)
    EEsettings[7] = wnd.txt7.GetLineText(0)
    EEsettings[8] = wnd.txt8.GetLineText(0)
    wnd.barStatus.SetStatusText("Wait...",0)
    for j in range(0,9): # 9 only goes up to 8??
        #print "sending ",EEsettings[j]," to ",j+16
        bus.i2c(i2cadr,[0x91,j+16,int(EEsettings[j])],0)
        sleep(0.1)
    # now soft reset
    onSoftreset(wnd)
    update_EEvalues(wnd)

# ------------------------------------------------------------------------------
# Global power
# ------------------------------------------------------------------------------
def onGlobal(wnd):
    power = wnd.sldGlobal.GetValue()
    #sv3(wnd.sp,wnd.adr+"g"+str(power)+"\r")
    bus.i2c(i2cadr,[4,power],0)
    

# ******************************************************************************
# DC Motor tab
# ******************************************************************************
# ------------------------------------------------------------------------------
# DC Motor A
# ------------------------------------------------------------------------------
def onDirectionA(wnd):
    direction = wnd.rdoDirectionA.GetSelection()
    power = int(wnd.sldPowerA.GetValue())
    s1 = power / 256
    s0 = power % 256
    #sv3(wnd.sp,wnd.adr+"a"+power+","+direction+"\r")
    bus.i2c(i2cadr,[1,s1,s0,direction],0)

# ------------------------------------------------------------------------------
# DC Motor B
# ------------------------------------------------------------------------------
def onDirectionB(wnd):
    direction = wnd.rdoDirectionB.GetSelection()
    power = int(wnd.sldPowerB.GetValue())
    s1 = power / 256
    s0 = power % 256
    #sv3(wnd.sp,wnd.adr+"b"+power+","+direction+"\r")
    bus.i2c(i2cadr,[2,s1,s0,direction],0)

# ******************************************************************************
# Stepper tab
# ******************************************************************************
# ------------------------------------------------------------------------------
# Go button
# ------------------------------------------------------------------------------
def onGo(wnd):
    list = []
    steps = int(wnd.txtSteps.GetLineText(0))
    speed = int(wnd.txtSpeed.GetLineText(0))
    direction = wnd.rdoDirection.GetSelection()
    s3 = steps / 16777216 #2^24
    s2 = steps / 65535 #2^16
    s1 = steps / 256 #2^8
    s0 = steps % 256 # remainder
    p1 = speed / 256
    p0 = speed % 256
    list = [9,s3,s2,s1,s0,p1,p0,direction]
    # change text on button to stop
    stepsToGo = '1'
    #if sv3(wnd.sp,wnd.adr+"s"+steps+","+speed+","+direction+"\r") != ACK:
    #    util_error("Problem writing step instruction\nCheck mode is 0")
    bus.i2c(i2cadr,list,0)
    # this is not a good way to see when the steps have stopped but 
    # it is the only way when only a serial connection is available
#    while stepsToGo != '0':
#        stepsToGo = sv3(wnd.sp,wnd.adr+"f\r")
#        wnd.lblStepper.SetLabel(stepsToGo)
#        sleep(0.3)
#        print stepsToGo


"""
class bv4603(wx.Frame):
    def __init__(self, type, wnd, com, *args, **kwds):
        # begin wxGlade: pi11.__init__
        kwds["style"] = wx.DEFAULT_FRAME_STYLE
        wx.Frame.__init__(self, *args, **kwds)

        if type == 'I':
            import bv4603_Ievents as events
        if type == 'S':
            import bv4603_events as events
        self.evt = events
        self.sp = com
        self.wnd = wnd
        self.adr = wnd.barStatus.GetStatusText(5)
      
        # Menu Bar

##binds
        self.Bind(wx.EVT_ACTIVATE, self.evt.onActivate(self))
        self.Bind(wx.EVT_CLOSE, self.onClose)

## events
    def onClose(self, event):  # wxGlade: bv4603.<event_handler>
        self.evt.onClose(self,event)

    def onSoftreset(self, event):  # wxGlade: bv4603.<event_handler>
        self.evt.onSoftreset(self)

    def onHardreset(self, event):  # wxGlade: bv4603.<event_handler>
        self.evt.onHardreset(self)

    def onEEupdate(self, event):  # wxGlade: bv4603.<event_handler>
        self.evt.onEEupdate(self)

    def onGlobal(self, event):  # wxGlade: bv4603.<event_handler>
        self.evt.onGlobal(self)

    def onDirectionA(self, event):  # wxGlade: bv4603.<event_handler>
        self.evt.onDirectionA(self)

    def onDirectionB(self, event):  # wxGlade: bv4603.<event_handler>
        self.evt.onDirectionB(self)

    def onGo(self, event):  # wxGlade: bv4603.<event_handler>
        self.evt.onGo(self)

"""
