#!/usr/bin/env python
#

import wx
import os
from time import sleep
import sys
sys.path.append("device")
from sv3common import sv3, util_error, util_information, Connect, writeIni
# Baud rates on sv3 devices are coded inro single digits
BAUDRATES = {2400:'1',4800:'2',9600:'3',14400:'4',19200:'5',38400:'6',57600:'7',115200:'8'}

ACK = '\006'
NACK = '\025'
IACK = '\076' # inverted ACK
PORT = "COM21"
BAUD = 115200

# bitmaps
bitmaps = {'Logo': "images/xLogo.ico", 'Refresh':"images/yRefesh.bmp", \
        'Connect':"images/Connected.ico", 'Disconnect':"images/Disconnected.ico" }

# contolling globals
bus = None # I2C bus for both nt and posix
address = "-1" # address of current device
EEdata = [] # keeps a list of changed addresses for EEPROM

# devices
from bv4601 import bv4601
from bv4603 import bv4603
from bv4614 import bv4614
from p011 import p011 # ADC
from p015 import p015 # GPIO
from p017 import p017 # LCD
Devices = {'406':bv4603,'4601':bv4601,'4603':bv4603,'4604':bv4603,'4614':bv4614,'11':p011, \
           '17':p017,'15':p015}
Descriptions = {'406':'This device is now the BV4604','4601':'Twin relay with ADC', \
                '4603':'Small Microstep motor driver','4604':'Larger Microstep motor driver', \
                '4614':'7 Segment LED Display','11':'ADC Chip','17':'LCD Display Chip', \
                '15':'GPIO with PWM'}


# ==============================================================================
# Part 2
# event handlers
# ==============================================================================

# ------------------------------------------------------------------------------
# on start up check to see if device and com port are already in place
# sv3.ini will exists if this has been run before
# ------------------------------------------------------------------------------
def onActivate(wnd):
    #connect to i2c bus
    global bus
    print os.name
    if os.name == 'nt':
        try:
            from bv4221_i2c import By2cbus
            bus = By2cbus(PORT,BAUD)
        except:
            util_error("Platform Windows, unable to open COM "+PORT+" to connect\nthe BV4221_V2")
        else:
            wnd.barStatus.SetStatusText("I2C Ready")
    if os.name == 'posix':
        try:
            from notsmb import notSMB
            bus = notSMB(0) # set default to i2c channel 0
        except:
            util_error("Platform POSIX, unable to open I2C bus 0\nHas notsmb been installed?")
        else:
            wnd.barStatus.SetStatusText("I2C Ready Bus 0")
            
            
    # remove com and baud
    wnd.barStatus.SetStatusText("",1)
    wnd.barStatus.SetStatusText("",2)
    # add buses
    wnd.cboBus.Clear()
    wnd.cboBus.Append('0')
    wnd.cboBus.Append('1')
    wnd.cboBus.SetSelection(0)
        

# ******************************************************************************
# EEPROM

# ------------------------------------------------------------------------------
# this is a tab change event. Idealy the tabshould be disabled until the
# user selects a device
# ------------------------------------------------------------------------------
def onEeprom(wnd):
    if len(wnd.txtI2caddress.GetValue()) > 0:
        print "okay"
    else:
        util_error("There is no I2C address")

# ------------------------------------------------------------------------------
# EEPROM refresh button
# Reads the whole of the EEPROM and loads it into the grid
# ------------------------------------------------------------------------------
def onEERefresh(wnd):
    elist = []
    wnd.barStatus.SetStatusText("Wait...")
    try: # could be specified as hex
        adr = int(wnd.txtI2caddress.GetValue())
    except:
        adr = int(wnd.txtI2caddress.GetValue(),16)
    for j in range(0,255):
        v = bus.i2c(adr,[0x90,j],1)
        elist.append(v[0])
    row=0
    col=0
    #for cell in elist:
    for j in range(0, len(elist)-1):
        hexval = format(int(elist[j]),'02x')
        wnd.grdEeprom.SetCellValue(row,col, hexval )
        wnd.grdEeprom.SetCellTextColour(row, col, wx.BLACK)
        #print row, col, j, elist[j]
        col = col + 1
        if col > 7:
            col = 0
            row = row + 1
    wnd.barStatus.SetStatusText("")

# ------------------------------------------------------------------------------
# When the user changesthe contents of the EEPROM grid, that locationis kept
# in EEdata so that the change can be written to later. This method is much
# quicker and less error prone than writing the whole EEPROM data each time.
# ------------------------------------------------------------------------------
def onCellchange(wnd,event):
    row = event.GetRow()
    col = event.GetCol()
    wnd.grdEeprom.SetCellTextColour(row, col, wx.RED)
    EEdata.append((row*8)+col)

# ------------------------------------------------------------------------------
# EEPROM write button
# takes changed data as given in the list EEdata and writes the contents
# of the cell to EEPROM
# ------------------------------------------------------------------------------
def onEEWrite(wnd):
    wnd.barStatus.SetStatusText("Wait...")
    try: # could be specified as hex
        adr = int(wnd.txtI2caddress.GetValue())
    except:
        adr = int(wnd.txtI2caddress.GetValue(),16)
    for j in EEdata:
        row = j / 8
        col = j % 8
        cell = wnd.grdEeprom.GetCellValue(row,col)
        #print j,row,col,cell
        wnd.grdEeprom.SetCellTextColour(row, col, wx.GREEN)
        # j is the address and v is the value
        vs = int(cell,16)
        # have to put data in high byte of word and value in low
#        wrd = (vs*256)+int(j)
#        bus.write_word_data(adr,0x91,wrd)
        bus.i2c(adr,[0x91,j,vs],0)
    wnd.grdEeprom.Refresh()
    wnd.barStatus.SetStatusText("Done...")

# ------------------------------------------------------------------------------
# [0]= id, [1]=firmware, [2]=description
# ------------------------------------------------------------------------------
def getInfo(addr):
    inf = []
    # device id
    devid = bus.i2c(addr,[0xa1],2)
    device = int(devid[0]) * 256
    device += int(devid[1])
    inf.append(str(device)) # device id
    # firmware version
    fw = bus.i2c(addr,[0xa0],2)
    inf.append(str(fw))
    # name
    try:
        inf.append(Descriptions[str(device)])
    except:
        inf.append("Unknown")
    return inf

# ------------------------------------------------------------------------------
# discover devices on the bus
# ------------------------------------------------------------------------------
def onDiscover(wnd):
    s = bus.detect()
    wnd.text_ctrl_1.Clear()
    wnd.text_ctrl_1.AppendText("Devices addresses discovered on this bus In Decimal:\r\n")
    if s:
        for n in s:
            inf = getInfo(n)
            wnd.text_ctrl_1.AppendText(str(n)+" ") # address
            wnd.text_ctrl_1.AppendText(inf[0]+" ")
            wnd.text_ctrl_1.AppendText(inf[1]+" ")
            wnd.text_ctrl_1.AppendText(inf[2]+"\r\n")

# ------------------------------------------------------------------------------
# Activates device window
# Calls window dependig on value in status bar, adding windows needs import
# and device dictionary updating
# ------------------------------------------------------------------------------
def onOpen(wnd):
    textI2c = wnd.txtI2caddress.GetLineText(0)
    if len(textI2c) < 1:
        util_error("Put an I2C address in the text box")
        return
        
    i2cadr =  int(textI2c,0) # get 7 bit i2c address
    wnd.barStatus.SetStatusText(str(i2cadr),5) # I2c address
    inf = getInfo(i2cadr)
    # update status bar
    wnd.barStatus.SetStatusText(inf[0],3) # ID
    wnd.barStatus.SetStatusText(inf[1],4) # FW
    wnd.barStatus.SetStatusText(inf[2]) #Description

    # sp is now used for i2c bus
    sp = bus
#   try:
    frame_2 = Devices[inf[0]]('I', wnd, sp, None, wx.ID_ANY)
    frame_2.Show()
    frame_2.MakeModal(True)
#    except:
#        util_error("Device ID is not valid")

# ------------------------------------------------------------------------------
# change bus number
# ------------------------------------------------------------------------------
def onBus(wnd):
    global bus
    try:
        from notsmb import notSMB
        bn = int(wnd.cboBus.GetValue());
        bus = notSMB(bn)
    except:
        util_error("Platform POSIX, unable to open I2C bus 0\nHas notsmb been installed?")
    else:
        wnd.barStatus.SetStatusText("I2C Ready Bus "+wnd.cboBus.GetValue())

# ------------------------------------------------------------------------------
# have not got this working yet
# ------------------------------------------------------------------------------
def onClose(wnd):
    if activated:
        #control.Destroy()
        if sp:
            sp.close()

#code to put in frame class
"""
Replace these: (function __init__)
(function properties)
        _icon.CopyFromBitmap(wx.Bitmap(bitmaps['Logo'], wx.BITMAP_TYPE_ANY))
        
        
        self.grdEeprom.SetRowLabelValue(0, _("0x00"))
        self.grdEeprom.SetRowLabelValue(1, _("0x08"))
        self.grdEeprom.SetRowLabelValue(2, _("0x10"))
        self.grdEeprom.SetRowLabelValue(3, _("0x18"))
        self.grdEeprom.SetRowLabelValue(4, _("0x20"))
        self.grdEeprom.SetRowLabelValue(5, _("0x28"))
        self.grdEeprom.SetRowLabelValue(6, _("0x30"))
        self.grdEeprom.SetRowLabelValue(7, _("0x38"))
        self.grdEeprom.SetRowLabelValue(8, _("0x40"))
        self.grdEeprom.SetRowLabelValue(9, _("0x48"))
        self.grdEeprom.SetRowLabelValue(10, _("0x50"))
        self.grdEeprom.SetRowLabelValue(11, _("0x58"))
        self.grdEeprom.SetRowLabelValue(12, _("0x60"))
        self.grdEeprom.SetRowLabelValue(13, _("0x68"))
        self.grdEeprom.SetRowLabelValue(14, _("0x70"))
        self.grdEeprom.SetRowLabelValue(15, _("0x78"))
        self.grdEeprom.SetRowLabelValue(16, _("0x80"))
        self.grdEeprom.SetRowLabelValue(17, _("0x88"))
        self.grdEeprom.SetRowLabelValue(18, _("0x90"))
        self.grdEeprom.SetRowLabelValue(19, _("0x98"))
        self.grdEeprom.SetRowLabelValue(20, _("0xA0"))
        self.grdEeprom.SetRowLabelValue(21, _("0xA8"))
        self.grdEeprom.SetRowLabelValue(22, _("0xB0"))
        self.grdEeprom.SetRowLabelValue(23, _("0xB8"))
        self.grdEeprom.SetRowLabelValue(24, _("0xC0"))
        self.grdEeprom.SetRowLabelValue(25, _("0xC8"))
        self.grdEeprom.SetRowLabelValue(26, _("0xD0"))
        self.grdEeprom.SetRowLabelValue(27, _("0xD8"))
        self.grdEeprom.SetRowLabelValue(28, _("0xE0"))
        self.grdEeprom.SetRowLabelValue(29, _("0xE8"))
        self.grdEeprom.SetRowLabelValue(30, _("0xF0"))
        self.grdEeprom.SetRowLabelValue(31, _("0xF8"))

Add these
        self.Bind(wx.EVT_ACTIVATE, sv3evt.onActivate(self))
#        self.Bind(wx.EVT_CLOSE, sv3utils.onClose(self)


    def ontxtI2caddress(self, event):  # wxGlade: MyFrame.<event_handler>
        sv3evt.onOpen(self)

    def onOpen(self, event):  # wxGlade: MyFrame.<event_handler>
        sv3evt.onOpen(self) # pass the list

    def onDiscover(self, event):  # wxGlade: MyFrame.<event_handler>
        sv3evt.onDiscover(self)

    def onEERefresh(self, event):  # wxGlade: MyFrame.<event_handler>
        sv3evt.onEERefresh(self)

    def onEEWrite(self, event):  # wxGlade: MyFrame.<event_handler>
        sv3evt.onEEWrite(self)

    def onEeprom(self, event):  # wxGlade: MyFrame.<event_handler>
        sv3evt.onEeprom(self)

    def onCellchange(self, event):  # wxGlade: MyFrame.<event_handler>
        sv3evt.onCellchange(self, event)


"""